"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const Docker = require("dockerode");
const BootstrapUtils_1 = require("./BootstrapUtils");
const fs_1 = require("fs");
const sleep_1 = require("./sleep");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class MongoService {
    constructor(params) {
        this.params = params;
    }
    async run() {
        const dir = `${process.cwd()}/${this.params.target}`;
        const mongoDbDir = `${dir}/data/mongo`;
        if (this.params.reset && fs_1.existsSync(mongoDbDir)) {
            logger.info('Resetting mongo db');
            BootstrapUtils_1.BootstrapUtils.deleteFolderRecursive(mongoDbDir);
        }
        await BootstrapUtils_1.BootstrapUtils.mkdir(`${this.params.target}/data/nemesis-data`);
        await BootstrapUtils_1.BootstrapUtils.mkdir(`${this.params.target}/data/mongo`);
        await BootstrapUtils_1.BootstrapUtils.mkdir(`${this.params.target}/state`);
        const docker = new Docker({ socketPath: '/var/run/docker.sock' });
        const cmd = ['bash', '-c', 'mongod --dbpath=/dbdata --bind_ip=db'];
        logger.info('Running mongo');
        const binds = [`${mongoDbDir}:/dbdata:rw`];
        const createOptions = {
            Hostname: 'db',
            HostConfig: {
                Binds: binds,
                PortBindings: { '27017/tcp': [{ HostPort: '27017' }] },
                AutoRemove: true,
                NetworkMode: 'symbol-network',
            },
        };
        const startOptions = {};
        docker.run('mongo', cmd, process.stdout, createOptions, startOptions);
        await sleep_1.sleep(1000);
        await this.initDb();
    }
    async initDb() {
        const workingDir = `${process.cwd()}`;
        const target = `${workingDir}/${this.params.target}`;
        if (this.params.reset && fs_1.existsSync(target)) {
            logger.info('Resetting mongo db');
            BootstrapUtils_1.BootstrapUtils.deleteFolderRecursive(target);
        }
        const docker = new Docker({ socketPath: '/var/run/docker.sock' });
        const cmd = [
            'bash',
            '-c',
            'apt-get install -y iputils-ping && ls -la /userconfig && ping db && /bin/bash /userconfig/mongors.sh && touch /state/mongo-is-setup && touch /state/mongo-is-setup',
        ];
        logger.info('Initializing mongo');
        logger.info(`${this.params.root}/config/docker/mongo`);
        const binds = [
            `${target}/data/mongo:/dbdata:rw`,
            `${target}/state:/state`,
            `${this.params.root}/config/docker/mongo/:/userconfig/:ro`,
        ];
        const createOptions = { HostConfig: { Binds: binds, AutoRemove: true, NetworkMode: 'symbol-network' } };
        const startOptions = {};
        await docker.run('mongo', cmd, process.stdout, createOptions, startOptions);
    }
}
exports.MongoService = MongoService;
